/*
Module : RASMONITOR.CPP
Purpose: Defines the implementation for a number of wrapper classes for 
         monitoring Ras connections 
Created: PJN / 18-06-1997
History: PJN / 28-09-1998 1.  Updated code to include a virtual OnDial method
                          2.  Optimized the code by using CArray::ElementAt
                          3.  Added the following member variables to CRasConnection:
                              Dial Time, whether or not the connection is currently
                              connected, Dial Duration and the RAS handle
                          4.  Renamed the connection variable of CRasConnection
                          5.  Updated the sample program to v1.1
                          6.  Updated the version info in the sample app to include 
                              the build date
                          7.  Fixed a problem when enumerating more than one ras
                              connection at a time.
                          8.  Sample app now uses LoadImage instead of LoadIcon to avoid
                              the shell first scaling up to 32*32 and then back to 16*16.
                              What this means is that the icons RasMan puts in the tray
                              notification area will not look blurred.
                          9.  Fixed a memory leak in CRasMonitor::DoCheck() when multiple
                              connections were being enumerated.
                          10. Included a new selection of tray icons in the sample app.
                          11. Added a popup menu item to bring up the help file.
         PJN / 28-10-1998 1.  Major optimisation added to reduce memory usage (c. 3 MB) 
                              on NT when Ras service is not running. Thanks to Daniel 
                              Harth for this very neat optimisation.
                          2.  Internally the code now uses a hidden window to perform 
                              the monitoring instead of a worker thread. This helps avoid 
                              potential deadlock problems which can occur as previously 
                              the code used a worker thread.
                          3.  Now uses window messages instead of virtual functions to 
                              implement class customisation
         PJN / 02-02-1999 1.  Fixed a compile problem which was occurring when code was 
                              compiled for UNICODE.
                          2.  Fixed a crash which was occuring when the code was run on NT 
                              and you do not have Ras installed.
                          3.  When running on NT, the code now only uses 
                              SC_MANAGER_ENUMERATE_SERVICE priviledges when connecting to 
                              the Service Configuration Manager.
                          4.  CRasConnection connection now includes a standard MFC 
                              Serialize method.
         PJN / 11-02-1999 1.  Now reports the following additional statistics:
                              a) Connection speed (Bytes / Second) (on Windows 95, 98 only)
                              b) Bytes sent (Windows 95, 98 & NT)
                              c) Bytes received (Windows 95, 98 & NT)
         PJN / 25-01-2000 1.  Removed a memory leak which was occuring on NT where the 
                              CPdhCounter instances in the GetInitialConnectionDetails and
                              GetCurrentConnectionDetails functions were not being tidied 
                              up. This has been accomplised by restructuring the code to 
                              create and add the counters to the performance once of in the
                              CRasConnection constructors.
         PJN / 07-05-2000 1.  Fixed a problem on NT where the PDH counters were being freed
                              prior to the being added to the m_Connections array
                          2.  Also now allows the PDH counter names to be customized via an advanced 
                              registry value.
         PJN / 30-04-2001 1.  Updated copyright messages.
                          2.  Fixed a problem where the code would not run correctly on Win 9x.
                          3.  Connection speed for a connection is now reported on via a call to
                              TAPI just like the way the SDK sample RasBaud operates.
                          4.  Provision of the following defines which allow fine grained control
                              over what API's RasMonitor pulls in:
                              RAS_MONITOR_NO_DYNDATA
                              RAS_MONITOR_NO_PDH
                              RAS_MONITOR_NO_TAPI
         PJN / 19-12-2001 1.  Code now tries to use RasGetConnectionDetails in preference to any
                              of the other methods. This new API is available from Windows 2000
                              onwards.
                          2.  Moved typedefs into namespace of class to avoid polluting the global
                              namespace
                          3.  Made all static data and methods non-static. This makes the code more
                              multi-thread safe in addition to the code being cleaner.
                          4.  Reworked how the NT Performance names can be tweaked by client apps
                          5.  Reviewed all the TRACE statements to ensure they were comprehensive
                              and accurate.
                          6.  Optimized checking of the Ras service state when running on NT.
                          7.  Removed need to keep an outstanding handle on the SCM throughout
                              the lifetime of the code when running on NT.
         PJN / 06-07-2003 1.  Now gets Caller ID number, Caller ID name, Called ID Number, Called Name, 
                              Connected ID number, Connected ID Name, Displayable Address, 
                              Called Party and comment fields from TAPI for each call.
                              

Copyright (c) 1998 - 2003 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


///////////////////////////// Includes ////////////////////////////////////////

#include "stdafx.h"
#include "dyndata.h"
#ifndef __AFXPRIV_H__
#pragma message("To avoid this message, please put afxpriv.h in your pre compiled header (normally stdafx.h)")
#include <afxpriv.h>
#endif
#include "rasmonitor.h"




///////////////////////////// Statics / Locals ////////////////////////////////

#ifdef _DEBUG
#undef THIS_FILE
#define new DEBUG_NEW
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif




///////////////////////////// Implementation //////////////////////////////////

CRasConnection::CRasConnection(CRasMonitor* pMonitor)
{
  //Call the helper constructor method
  Init(pMonitor);
}

CRasConnection::CRasConnection(CRasMonitor* pMonitor, const RASCONN& Connection) 
{
  //Call the helper constructor method
  Init(pMonitor);

  //Hive away all the other connection details
  m_sName       = Connection.szEntryName;
	m_sDeviceType = Connection.szDeviceType;
	m_sDeviceName = Connection.szDeviceName;
  m_hRasConn    = Connection.hrasconn;
  m_bConnected  = m_pMonitor->IsConnected(m_hRasConn);
}

void CRasConnection::Init(CRasMonitor* pMonitor)
{
  //Hive away a pointer to the main class instance
  ASSERT(pMonitor);
  m_pMonitor = pMonitor;

#ifndef RAS_MONITOR_NO_PDH
  m_pNTcounterR = NULL;
  m_pNTcounterT = NULL;
#endif
  ZeroMemory(&m_DialTime, sizeof(SYSTEMTIME));
  ZeroMemory(&m_ConnectionTime, sizeof(SYSTEMTIME));
  ZeroMemory(&m_HangupTime, sizeof(SYSTEMTIME));
  m_bConnected = FALSE;
  m_ConnectDuration = 0;
  m_DialDuration = 0;
  m_dwBytesReceived = 0;
  m_dwBytesSent = 0;        
  m_dwConnectionSpeed = 0;
  m_dwInitialBytesReceived = 0;
  m_dwInitialBytesSent = 0;
  m_hRasConn = NULL;

  if (m_pMonitor->m_pfnRasGetConnectionStatistics == NULL)
  {
    //Setup the NT performance counters. Note that NT performance counters 
    //currently do not support the connection speed
  #ifndef RAS_MONITOR_NO_PDH
    ASSERT(m_pMonitor);
    if (m_pMonitor->RunningNT())
    {
      try
      {
        //Allocate the counters
        m_pNTcounterR = new CPdhCounter(m_pMonitor->GetPDHTotalBytesRxCounterName());
        m_pNTcounterT = new CPdhCounter(m_pMonitor->GetPDHTotalBytesTxCounterName());

        //Open the query
        m_NTquery.Open();

        //Add the counters
        m_NTquery.Add(*m_pNTcounterR);
        m_NTquery.Add(*m_pNTcounterT);
      }
      catch (CPdhException* pEx)
      {
        TRACE(_T("CRasConnection::Init, An error occurred setting up the performance values on NT, Error:%x\n"), pEx->m_Status);
		    pEx->Delete();
      }
    }
  #endif
  }
}

CRasConnection::~CRasConnection()
{
  ASSERT(m_pMonitor);
  if (m_pMonitor->m_pfnRasGetConnectionStatistics == NULL)
  {
    //Free up the NT performance counter if we are running on NT
  #ifndef RAS_MONITOR_NO_PDH
    ASSERT(m_pMonitor);
    if (m_pMonitor->RunningNT())
    {
      try
      {
        //Remove the counters
        m_NTquery.Remove(*m_pNTcounterR);
        m_NTquery.Remove(*m_pNTcounterT);

        //Close the query
        m_NTquery.Close();

        //Free up the heap memory
        if (m_pNTcounterR)
        {
          delete m_pNTcounterR;
          m_pNTcounterR = NULL;
        }
        if (m_pNTcounterT)
        {
          delete m_pNTcounterT;
          m_pNTcounterT = NULL;
        }
      }
      catch (CPdhException* pEx)
      {
        TRACE(_T("CRasConnection::~CRasConnection, An error occurred freeing up the performance values on NT, Error:%x\n"), pEx->m_Status);
		    pEx->Delete();
      }
    }
  #endif
  }
}

void CRasConnection::GetInitialConnectionDetails()
{
  ASSERT(m_pMonitor);

  //Try to use the "RasGetConnectionStatistics" SDK call 
  //if it is available. Using this function is a whole
  //lot simpler than all the other hoops we have to go 
  //thro if it is not available.
  if (m_pMonitor->m_pfnRasGetConnectionStatistics)
    GetDetailsViaRasGetConnectionStatistics();
  else
  {
    if (m_pMonitor->RunningNT())
    {
    #ifndef RAS_MONITOR_NO_PDH
      //Note that NT performance counters currently does not support the connection speed
      try
      {
        //Collect the data and interpret
        m_NTquery.Collect();

        //Interpret the results
        PDH_FMT_COUNTERVALUE Received;
        m_pNTcounterR->GetFormattedValue(PDH_FMT_LONG, Received);
        if (Received.CStatus == PDH_CSTATUS_VALID_DATA)
          m_dwInitialBytesReceived = (DWORD) Received.longValue;
        else
          TRACE(_T("CRasConnection::GetInitialConnectionDetails, Failed in call to get formatted value for Bytes received, Error:%x\n"), Received.CStatus);

        PDH_FMT_COUNTERVALUE Txmitted;
        m_pNTcounterT->GetFormattedValue(PDH_FMT_LONG , Txmitted);
        if (Txmitted.CStatus == PDH_CSTATUS_VALID_DATA)
          m_dwInitialBytesSent = (DWORD) Txmitted.longValue;
        else
          TRACE(_T("CRasConnection::GetInitialConnectionDetails, Failed in call to get formatted value for Bytes sent, Error:%x\n"), Txmitted.CStatus);
      }
      catch (CPdhException* pEx)
      {
        TRACE(_T("CRasConnection::GetInitialConnectionDetails, An error occurred get the performance values on NT, Error:%x\n"), pEx->m_Status);
		    pEx->Delete();
      }
    #endif
    }
    else
    {
    #ifndef RAS_MONITOR_NO_DYNDATA
      //Collect the Initial Bytes received
      CDynDataCounter counter;
      CString sCounter = m_pMonitor->GetDynDataCounterName();
      if (counter.Start(sCounter, m_pMonitor->GetDynDataTotalBytesRecvdCounterName()))
        if (!counter.Collect(m_dwInitialBytesReceived))
          TRACE(_T("CRasConnection::GetInitialConnectionDetails, Failed to retrieve the initial bytes received\n"));

      //Collect the Initial Bytes sent
      if (counter.Start(sCounter, m_pMonitor->GetDynDataTotalBytesSentCounterName()))
        if (!counter.Collect(m_dwInitialBytesSent))
          TRACE(_T("CRasConnection::GetInitialConnectionDetails, Failed to retrieve the initial bytes sent\n"));
    #endif
    }
  }

//Get all the details from TAPI
#ifndef RAS_MONITOR_NO_TAPI
  m_pMonitor->GetRasTAPIDetails(m_sDeviceName, (m_pMonitor->m_pfnRasGetConnectionStatistics != NULL), *this);
#endif
}

void CRasConnection::GetDetailsViaRasGetConnectionStatistics()
{
  CRasMonitor::RAS_STATS rs;
  rs.dwSize = sizeof(CRasMonitor::RAS_STATS);
  DWORD dwSuccess = m_pMonitor->m_pfnRasGetConnectionStatistics(m_hRasConn, &rs);
  if (dwSuccess == ERROR_SUCCESS)
  {
    //Hive away all the values we are interested in
    m_dwBytesSent = rs.dwBytesXmited;
    m_dwBytesReceived = rs.dwBytesRcved;
    m_dwConnectionSpeed = rs.dwBps;
    m_ConnectDuration = rs.dwConnectDuration / 1000;
  }
  else
    TRACE(_T("CRasConnection::GetCurrentConnectionDetails, Failed in call to RasGetConnectionStatistics, Error:%d\n"), GetLastError());
}

void CRasConnection::GetCurrentConnectionDetails()
{
  ASSERT(m_pMonitor);

  //Try to use the "RasGetConnectionStatistics" SDK call 
  //if it is available. Using this function is a whole
  //lot simpler than all the other hoops we have to go 
  //thro if it is not available.
  if (m_pMonitor->m_pfnRasGetConnectionStatistics)
    GetDetailsViaRasGetConnectionStatistics();
  else
  {
    if (m_pMonitor->RunningNT())
    {
    #ifndef RAS_MONITOR_NO_PDH
      //Note that NT performance counters currently do not support the connection speed
      try
      {
        //Collect the data and interpret
        m_NTquery.Collect();

        //Interpret the results
        PDH_FMT_COUNTERVALUE Received;
        m_pNTcounterR->GetFormattedValue(PDH_FMT_LONG, Received);
        if (Received.CStatus == PDH_CSTATUS_VALID_DATA)
          m_dwBytesReceived = ((DWORD) Received.longValue) - m_dwInitialBytesReceived;
        else
          TRACE(_T("CRasConnection::GetCurrentConnectionDetails, Failed in call to get formatted value for Bytes received\n"));

        PDH_FMT_COUNTERVALUE Txmitted;
        m_pNTcounterT->GetFormattedValue(PDH_FMT_LONG , Txmitted);
        if (Txmitted.CStatus == PDH_CSTATUS_VALID_DATA)
          m_dwBytesSent = ((DWORD) Txmitted.longValue) - m_dwInitialBytesSent;
        else
          TRACE(_T("CRasConnection::GetCurrentConnectionDetails, Failed in call to get formatted value for Bytes sent\n"));
      }
      catch(CPdhException* pEx)
      {
        TRACE(_T("CRasConnection::GetCurrentConnectionDetails, An error occurred get the performance values on NT, Error:%x\n"), pEx->m_Status);
		    pEx->Delete();
      }
    #endif
    }
    else
    {
    #ifndef RAS_MONITOR_NO_DYNDATA
      //Collect the Bytes received
      CDynDataCounter counter;
      DWORD dwData;
      CString sCounter = m_pMonitor->GetDynDataCounterName();
      if (counter.Start(sCounter, m_pMonitor->GetDynDataTotalBytesRecvdCounterName()))
        if (counter.Collect(dwData))
          m_dwBytesReceived = dwData - m_dwInitialBytesReceived;

      //Collect the Bytes sent
      if (counter.Start(sCounter, m_pMonitor->GetDynDataTotalBytesSentCounterName()))
        if (counter.Collect(dwData))
          m_dwBytesSent = dwData - m_dwInitialBytesSent;
    #endif  
    }
  }

//Get all the details from TAPI
#ifndef RAS_MONITOR_NO_TAPI
  m_pMonitor->GetRasTAPIDetails(m_sDeviceName, (m_pMonitor->m_pfnRasGetConnectionStatistics != NULL), *this);
#endif
}

void CRasConnection::SerializeTime(CArchive& ar, SYSTEMTIME& st)
{
  if (ar.IsLoading())
  {
    //Version stamp
    WORD wVersion;
    ar >> wVersion;

    //the actual data
    ar >> st.wYear;
    ar >> st.wMonth;
    ar >> st.wDayOfWeek;
    ar >> st.wDay;
    ar >> st.wHour;
    ar >> st.wMinute;
    ar >> st.wSecond;
    ar >> st.wMilliseconds;
  }
  else
  {
    //Version stamp the data
    WORD wVersion = 0x100;
    ar << wVersion;

    //the actual data
    ar << st.wYear;
    ar << st.wMonth;
    ar << st.wDayOfWeek;
    ar << st.wDay;
    ar << st.wHour;
    ar << st.wMinute;
    ar << st.wSecond;
    ar << st.wMilliseconds;
  }
}

void CRasConnection::Serialize(CArchive& ar)
{
  if (ar.IsLoading())
  {
    //Version stamp
    WORD wVersion;
    ar >> wVersion;

    ar >> m_sName;
    ar >> m_sDeviceName;
    ar >> m_sDeviceType;
    SerializeTime(ar, m_ConnectionTime);
    SerializeTime(ar, m_DialTime);
    ar >> m_bConnected;
    ar >> m_ConnectDuration;
    ar >> m_DialDuration;
    SerializeTime(ar, m_HangupTime);
    ar >> m_dwBytesReceived;
    ar >> m_dwBytesSent;        
    ar >> m_dwConnectionSpeed;

    if (wVersion > 0x100)
    {
      ar >> m_sCallerIDNumber;
      ar >> m_sCallerIDName;
      ar >> m_sCalledIDNumber;
      ar >> m_sCalledIDName;
      ar >> m_sConnectedIDNumber;
      ar >> m_sConnectedIDName;
      ar >> m_sDisplayableAddress;
      ar >> m_sCalledParty; 
      ar >> m_sComment;
    }
  }
  else
  {
    //Version stamp the data
    WORD wVersion = 0x101;
    ar << wVersion;

    ar << m_sName;
    ar << m_sDeviceName;
    ar << m_sDeviceType;
    SerializeTime(ar, m_ConnectionTime);
    SerializeTime(ar, m_DialTime);
    ar << m_bConnected;
    ar << m_ConnectDuration;
    ar << m_DialDuration;
    SerializeTime(ar, m_HangupTime);
    ar << m_dwBytesReceived;
    ar << m_dwBytesSent;        
    ar << m_dwConnectionSpeed;

    ar << m_sCallerIDNumber;
    ar << m_sCallerIDName;
    ar << m_sCalledIDNumber;
    ar << m_sCalledIDName;
    ar << m_sConnectedIDNumber;
    ar << m_sConnectedIDName;
    ar << m_sDisplayableAddress;
    ar << m_sCalledParty; 
    ar << m_sComment;
  }
}







CRasMonitor::CRasMonitor()
{
  //Initialize everything to default values
  m_pNotificationWnd = NULL;
  m_nNotifyMessage = 0;
  m_bStarted = FALSE;
  m_bArchivedDataLoaded = FALSE;
  m_hRasApi32 = NULL;
  m_nTimerID = 0;
  m_pfnRasEnumConnections = NULL;
  m_pfnRasGetConnectStatus = NULL;
  m_pfnRasGetConnectionStatistics = NULL;
  m_hLineApp = NULL;
  ZeroMemory(&m_archiveTime, sizeof(SYSTEMTIME));

  // Check for Windows NT
  OSVERSIONINFO osi;
  osi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osi);
  m_bRunningNT = (osi.dwPlatformId == VER_PLATFORM_WIN32_NT);

  //Initialize TAPI
#ifndef RAS_MONITOR_NO_TAPI
  InitializeTapi();
#endif

  if (m_bRunningNT)
  {
    // Open the Service control manager
    SC_HANDLE hSCM = OpenSCManager(NULL, NULL, SC_MANAGER_ENUMERATE_SERVICE);
    if (hSCM == NULL)
    {
      TRACE(_T("CRasMonitor::CRasMonitor, Can't open Service Control Manager, Error:%d\n"), GetLastError());
      return;
    }

    //Open up the RasMan service from the SCM
    m_hRasMan = OpenService(hSCM, _T("RasMan"), SERVICE_QUERY_STATUS);

    //Release the SCM handle now that we are finished with it
		CloseServiceHandle(hSCM);

    //If we failed to open the service, then log the error
    if (m_hRasMan == NULL)
    {
      TRACE(_T("CRasMonitor::CRasMonitor, Can't open RasMan service, Error:%d\n"), GetLastError());
      return;
    }

  }
}

CRasMonitor::~CRasMonitor()
{
  Stop();

  //Release the RasMan service Handle
	if (m_hRasMan)
	{
		CloseServiceHandle(m_hRasMan);
    m_hRasMan = NULL;
	}

  //Free up the Rasapi32 handle
	if (m_hRasApi32)
	{
		FreeLibrary(m_hRasApi32);
		m_hRasApi32 = NULL;
	}

  //Close down TAPI
#ifndef RAS_MONITOR_NO_TAPI
  TerminateTapi();
#endif
}

IMPLEMENT_DYNAMIC(CRasMonitor, CWnd)

BEGIN_MESSAGE_MAP(CRasMonitor, CWnd)
	//{{AFX_MSG_MAP(CRasMonitor)
	ON_WM_CREATE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CRasMonitor::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  //Let the parent do its thing
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  //create a 1 second timer
	m_nTimerID = SetTimer(GetTimerID(), 1000, NULL);
	
	return 0;
}

void CRasMonitor::OnTimer(UINT /*nIDEvent*/) 
{
  //Perform the check
  DoCheck(FALSE);
}

BOOL CRasMonitor::Start(CWnd* pNotifyWnd, UINT nNotifyMessage)
{
  ASSERT(!m_bStarted);

  //Store away the notification window
  ASSERT(pNotifyWnd && ::IsWindow(pNotifyWnd->GetSafeHwnd()));
  m_pNotificationWnd = pNotifyWnd;

  //Store away the notification message
  m_nNotifyMessage = nNotifyMessage;

  //create the hidden window which will do the monitoring
  CRect r(0, 0, 10, 10);
  if (!CreateEx(0, AfxRegisterWndClass(0), _T("CRasMonitor Notification Window"), WS_OVERLAPPED, r, NULL, 0))
  {
    TRACE(_T("Failed in creation of hidden Ras monitoring window, Error:%d\n"), GetLastError());
    return FALSE;
  }

  m_bStarted = TRUE;

  return TRUE;
}

BOOL CRasMonitor::Stop()
{
  if (m_bStarted)
  {
    //Final check since we are closing
    DoCheck(TRUE);

    //kill the timer
    KillTimer(m_nTimerID);

    //Destroy the hidden monitoring window
    DestroyWindow();

    m_pNotificationWnd = NULL;
    m_nNotifyMessage = 0;

    m_bStarted = FALSE;
  }

  //Free up the connections array
  for (int i=0; i<m_Connections.GetSize(); i++)
    delete m_Connections.GetAt(i);
  m_Connections.SetSize(0);

  return TRUE;
}

BOOL CRasMonitor::ConnectionExists(const CString& sName)
{
  return (GetConnectionIndex(sName) != -1);
}

BOOL CRasMonitor::IsConnected(HRASCONN hConnection)
{
  RASCONNSTATUS rcs;
  rcs.dwSize = sizeof(RASCONNSTATUS);
  if (RasGetConnectStatus(hConnection, &rcs) == 0)
    return (rcs.rasconnstate == RASCS_Connected);
  else
    return FALSE;
}

int CRasMonitor::GetConnectionIndex(const CString& sName)
{
	for (int i=0; i<m_Connections.GetSize(); i++)
	{
		if (m_Connections.GetAt(i)->m_sName == sName)
			return i;
	}

	return -1;
}

void CRasMonitor::OnConnect(const CRasConnection& connection)
{ 
  //Display some debug info
  TRACE(_T("CRasMonitor::OnConnect, %s has just connected, Dial Duration was %d\n"), connection.m_sName, connection.m_DialDuration);

  //Inform the notify window of the event
  ASSERT(m_pNotificationWnd);
  ASSERT(m_pNotificationWnd->GetSafeHwnd());
  m_pNotificationWnd->SendMessage(m_nNotifyMessage, RAS_CONNECT_EVENT, (LPARAM) &connection);
}

void CRasMonitor::OnDial(const CRasConnection& connection)
{
  //Display some debug info
  TRACE(_T("CRasMonitor::OnDial, %s has just dialled\n"), connection.m_sName);

  //Inform the notify window of the event
  ASSERT(m_pNotificationWnd);
  ASSERT(m_pNotificationWnd->GetSafeHwnd());
  m_pNotificationWnd->SendMessage(m_nNotifyMessage, RAS_DIAL_EVENT, (LPARAM) &connection);
}

void CRasMonitor::OnDisconnect(const CRasConnection& connection)
{
  //Display some debug info
  TRACE(_T("CRasMonitor::OnDisconnect, %s has just disconnected, Dial Duration was %d, Connection Time was %d\n"), connection.m_sName, connection.m_DialDuration, connection.m_ConnectDuration);

  //Inform the notify window of the event
  ASSERT(m_pNotificationWnd);
  ASSERT(m_pNotificationWnd->GetSafeHwnd());
  m_pNotificationWnd->SendMessage(m_nNotifyMessage, RAS_DISCONNECT_EVENT, (LPARAM) &connection);
}

void CRasMonitor::OnCheck(BOOL bClosing, RASCONN* pConnections, DWORD dwConnections)
{
  //Get the current time
	SYSTEMTIME now;
	GetLocalTime(&now);

	//Iterate through all the previously active connections
	//which has now disappeared and call the virtual OnDisconnect
	//function for that connection
	for (int i=m_Connections.GetUpperBound(); i>=0; i--)
	{
    BOOL bFound = FALSE;
    for (DWORD j=0; j<dwConnections && !bFound; j++)
      bFound = m_Connections.GetAt(i)->m_sName == pConnections[j].szEntryName;

    if (!bFound)
		{
      CRasConnection* pConnection = m_Connections.GetAt(i);
			SYSTEMTIME st;
      //If we are dealing with archived data then use the archive time stamp and
      //not the current time
      if (m_bArchivedDataLoaded)
        CopyMemory(&st, &m_archiveTime, sizeof(SYSTEMTIME));
      else  
			  CopyMemory(&st, &now, sizeof(SYSTEMTIME));
      if (pConnection->m_bConnected)
        pConnection->m_ConnectDuration = (CTime(st) - CTime(pConnection->m_ConnectionTime)).GetTotalSeconds();
      else
        pConnection->m_DialDuration = (CTime(st) - CTime(pConnection->m_DialTime)).GetTotalSeconds();
      pConnection->m_bConnected = FALSE;
      CopyMemory(&pConnection->m_HangupTime, &st, sizeof(SYSTEMTIME));

      //Call the virtual function
      OnDisconnect(*pConnection);

      //Free up the memory allocated for the connection
      delete m_Connections.GetAt(i);
      m_Connections.RemoveAt(i);
		}
	}

  //Iterate through all the current connections
  for (i=0; i<(int)dwConnections; i++)
  {
    int nConnection = GetConnectionIndex(pConnections[i].szEntryName);
    if (nConnection == -1)
    {
    	//For any new active connections which have appeared call the 
      //OnDial or the OnConnect function
      CRasConnection* pNewConnection = new CRasConnection(this, pConnections[i]);

      //Set the dial time
      CopyMemory(&pNewConnection->m_DialTime, &now, sizeof(SYSTEMTIME));

      //Call OnConnect or OnDial depending on the call state
      if (pNewConnection->m_bConnected)
      {
        CopyMemory(&pNewConnection->m_ConnectionTime, &now, sizeof(SYSTEMTIME));
        pNewConnection->m_DialDuration = 0;
        pNewConnection->GetInitialConnectionDetails();
        OnConnect(*pNewConnection);
      }
      else
      {
        //And call the OnDial method
        OnDial(*pNewConnection);
      }

      //Add to the array
      m_Connections.Add(pNewConnection);
    }
    else
    {
      CRasConnection* pConnection = m_Connections.GetAt(nConnection);

      //For existing connections, check to see if their connection state
      //has changed to connected and call the OnConnect function, Also call 
      //the GetInitialConnectionDetails method of CRasConnection
      if (IsConnected(pConnections[i].hrasconn))
      {
        if (!pConnection->m_bConnected)
        {
          CopyMemory(&pConnection->m_ConnectionTime, &now, sizeof(SYSTEMTIME));
          pConnection->m_DialDuration = (CTime(now) - CTime(pConnection->m_DialTime)).GetTotalSeconds();
          pConnection->m_bConnected = TRUE;
          pConnection->GetInitialConnectionDetails();
          OnConnect(*pConnection); 
        }
        else
          pConnection->GetCurrentConnectionDetails();
      }
    }

    if (bClosing)
    {
      //Call the Disconnect for all connections as we are about to finish
      //our monitoring
      CRasConnection* pConnection = m_Connections.GetAt(i);
      if (pConnection->m_bConnected)
        pConnection->m_ConnectDuration = (CTime(now) - CTime(pConnection->m_ConnectionTime)).GetTotalSeconds();
      else
        pConnection->m_DialDuration = (CTime(now) - CTime(pConnection->m_DialTime)).GetTotalSeconds();
      pConnection->m_bConnected = FALSE;
      CopyMemory(&pConnection->m_HangupTime, &now, sizeof(SYSTEMTIME));
      OnDisconnect(*pConnection);

      //Free up the memory associated with the connection
      delete m_Connections.GetAt(i);
      m_Connections.RemoveAt(i);
    }
  }

  //Now give the notification window a chance of updating ifself
  ASSERT(m_pNotificationWnd);
  ASSERT(m_pNotificationWnd->GetSafeHwnd());
  m_pNotificationWnd->SendMessage(m_nNotifyMessage, RAS_CHECK_EVENT, (LPARAM) &m_Connections);
}

void CRasMonitor::DoCheck(BOOL bClosing)
{
	//First work out how many connections there are currently
	DWORD dwConnections;
	RASCONN rasconn;
	rasconn.dwSize = sizeof(RASCONN);
	DWORD dwSize = rasconn.dwSize;
	DWORD dwSuccess = RasEnumConnections(&rasconn, &dwSize, &dwConnections);

	RASCONN* pRasConn = NULL;
	RASCONN* pActiveConnections = NULL;
	DWORD dwActiveConnections = 0;
	if (dwSuccess == 0)
	{
		if (dwConnections)
		{
			//At this stage we have a single active RASCONN
		  pActiveConnections = &rasconn;
		  dwActiveConnections = dwConnections;
		}
	}
	else if (dwSuccess != ERROR_BUFFER_TOO_SMALL)
	  return;
	else
	{
		//Allocate enough memory to satisy the request
		pRasConn = new RASCONN[dwConnections];
		pRasConn[0].dwSize = sizeof(RASCONN);
		dwSize = dwConnections*sizeof(RASCONN);

		//Now enumerate all the connections
		dwSuccess = RasEnumConnections(pRasConn, &dwSize, &dwConnections);

		//Some unknown error
		if (dwSuccess != 0)
		{
			delete [] pRasConn;
			return;
		}

		//At this stage we have an array of RASCONN which
		//are currently active
		pActiveConnections = pRasConn;
		dwActiveConnections = dwConnections;
	}

	//Call the virtual function to allow client customisation
  OnCheck(bClosing, pActiveConnections, dwActiveConnections);

	//Clean up any heap memory we may have used
	if (pRasConn)
	  delete [] pRasConn;
}

BOOL CRasMonitor::GetRasDLLEntryPoints()
{
  //Assume the worst
  BOOL bSuccess = FALSE;

  if (m_bRunningNT && (m_hRasApi32 == NULL))
  {
    //If the Ras service is opened, check that the service
    //is running before we load the RAS api dll
	  if (m_hRasMan)
	  {
      SERVICE_STATUS stat;
      QueryServiceStatus(m_hRasMan, &stat);
      if (stat.dwCurrentState != SERVICE_RUNNING) // not running?
			  return FALSE;
    }
  }

  //If the Ras dll is not already loaded, then load it up
	if (m_hRasApi32 == NULL)
	{
		m_hRasApi32 = LoadLibrary(_T("RASAPI32.DLL"));
		if (m_hRasApi32 == NULL)
		{
			TRACE(_T("CRasMonitor::GetRasDllEntryPoints, Can't load RASAPI32.DLL, Error:%d\n"), GetLastError());
			return FALSE;
		}

    //Then get the function entry points which we need
#ifndef UNICODE
		m_pfnRasEnumConnections = (PFNRASENUMCONNECTIONS) GetProcAddress(m_hRasApi32, "RasEnumConnectionsA");
		m_pfnRasGetConnectStatus = (PFNRASGETCONNECTSTATUS) GetProcAddress(m_hRasApi32, "RasGetConnectStatusA");
#else
		m_pfnRasEnumConnections = (PFNRASENUMCONNECTIONS) GetProcAddress(m_hRasApi32, "RasEnumConnectionsW");
		m_pfnRasGetConnectStatus = (PFNRASGETCONNECTSTATUS) GetProcAddress(m_hRasApi32, "RasGetConnectStatusW");
#endif

    //Also try to get the RasGetConnectionStatistics function (which makes the work of CRasMonitor a whole lot easier!
    m_pfnRasGetConnectionStatistics = (PFNRASGERCONNECTIONSTATISTICS) GetProcAddress(m_hRasApi32, "RasGetConnectionStatistics");
    if (m_pfnRasGetConnectionStatistics)
      TRACE(_T("CRasMonitor::GetRasDllEntryPoints, RasGetConnectionStatistics function found, Life is sweet<g>\n"));
    else
      TRACE(_T("CRasMonitor::GetRasDllEntryPoints, Cound not find RasGetConnectionStatistics function, Reverting to using TAPI, NT/Win2k/XP PDH and Win9x PerfData API's\n"));

    //In the unlikely event that either or both of the 2 critical functions we need fail, then cleanup gracefully
    if (m_pfnRasEnumConnections == NULL || m_pfnRasGetConnectStatus == NULL)
    {
			TRACE(_T("CRasMonitor::GetRasDllEntryPoints, Can't get RASAPI32.DLL entry points, Error:%d\n"), GetLastError());

      //Release our Rasapi handle
			FreeLibrary(m_hRasApi32);
			m_hRasApi32 = NULL;
      m_pfnRasGetConnectStatus = NULL;
      m_pfnRasEnumConnections = NULL;

			return 0;
    }
    else
      bSuccess = TRUE;
	}
  else
    bSuccess = TRUE;

  return bSuccess;
}

DWORD CRasMonitor::RasEnumConnections(LPRASCONN lprasconn, LPDWORD lpcb, LPDWORD lpcConnections)
{
  if (!GetRasDLLEntryPoints())
  {
		*lpcb = 0;
		*lpcConnections = 0;
		return 0;
	}

  VERIFY(m_pfnRasEnumConnections);
	DWORD dwRet = m_pfnRasEnumConnections(lprasconn, lpcb, lpcConnections);

	return dwRet;
}

DWORD CRasMonitor::RasGetConnectStatus(HRASCONN hrasconn, LPRASCONNSTATUS lprasconnstatus)
{
  if (!GetRasDLLEntryPoints())
  {
		ZeroMemory(lprasconnstatus, sizeof(RASCONNSTATUS));
		lprasconnstatus->dwSize=sizeof(RASCONNSTATUS);
		lprasconnstatus->rasconnstate=RASCS_Disconnected;

		return 0;
  }

  VERIFY(m_pfnRasGetConnectStatus);
	return m_pfnRasGetConnectStatus(hrasconn, lprasconnstatus);
}

void CRasMonitor::Serialize(CArchive& ar)
{
  if (ar.IsLoading())
  {
    //Version stamp the data
    WORD wVersion;
    ar >> wVersion;

    //the actual data
    int nSize;
    ar >> nSize;
    m_Connections.SetSize(nSize);
    for (int i=0; i<nSize; i++)
    {
      CRasConnection* pConnection = new CRasConnection(this);
      pConnection->Serialize(ar);
      m_Connections.SetAt(i, pConnection);
    }

    //Don't forget to read in the archive time
    CRasConnection::SerializeTime(ar, m_archiveTime);

    //Call the OnCheck method with the archived data
    m_bArchivedDataLoaded = TRUE;
    OnCheck(FALSE, NULL, 0);
    m_bArchivedDataLoaded = FALSE;
  }
  else
  {
    //Version stamp the data
    WORD wVersion = 0x100;
    ar << wVersion;

    //the actual data
    int nSize = m_Connections.GetSize();
    ar << nSize;
    for (int i=0; i<nSize; i++)
    {
      CRasConnection* pConnection = m_Connections.GetAt(i);
      pConnection->Serialize(ar);
    }

    //Archive out the current time aswell.
 		SYSTEMTIME archiveTime;
		GetLocalTime(&archiveTime);
    CRasConnection::SerializeTime(ar, archiveTime);
  }
}

#ifndef RAS_MONITOR_NO_TAPI
BOOL CRasMonitor::InitializeTapi()
{
  USES_CONVERSION;

  //Initialize TAPI
  long lReturn = lineInitialize(&m_hLineApp, AfxGetInstanceHandle(), lineCallbackFunc, T2A((LPTSTR) AfxGetAppName()), &m_dwNumTapiDevs); 
  if (lReturn)
  {
    TRACE(_T("CRasMonitor::InitializeTapi, lineInitialize failed, Error:%x\n"), lReturn);
    return FALSE;
  }
  return TRUE;
}

void CRasMonitor::TerminateTapi()
{
  //Close down TAPI
  lineShutdown(m_hLineApp);  // closing all calls and lines here also.
  m_hLineApp = NULL;
}

BOOL CRasMonitor::GetRasTAPIDetails(const CString& sDeviceName, BOOL bGetConnectionSpeed, CRasConnection& connection)
{
  //By default assume a baud rate of 0
  if (bGetConnectionSpeed)
    connection.m_dwConnectionSpeed = 0;

  //Allocate some structures we will need
  DWORD dwSize = sizeof(LINECALLLIST) + 128;
  LPLINECALLLIST lpCallList = (LPLINECALLLIST) new BYTE[dwSize];
  lpCallList->dwTotalSize = dwSize;
  dwSize = sizeof(LINEDEVCAPS) + 1024;
  LPLINEDEVCAPS lpLineDevCaps = (LPLINEDEVCAPS) new BYTE[dwSize];
  lpLineDevCaps->dwTotalSize = dwSize;

  //Iterate thro all the line devices
  HCALL hCall = 0;
  for (DWORD dwDeviceID=0; dwDeviceID<m_dwNumTapiDevs; dwDeviceID++)
  {
    DWORD dwApiVersion;
    LINEEXTENSIONID ExtensionID;
    long lReturn = lineNegotiateAPIVersion(m_hLineApp, dwDeviceID, TAPI_CURRENT_VERSION, TAPI_CURRENT_VERSION, &dwApiVersion, &ExtensionID);
    if (lReturn)
    {
      // This line device isn't useable by RAS anyway.
      continue;
    }

    //Get the name of this TAPI line device.
    while (TRUE)
    {
      lReturn = lineGetDevCaps(m_hLineApp, dwDeviceID, dwApiVersion, 0, lpLineDevCaps);
      if (lReturn)
      {
        TRACE(_T("CRasMonitor::GetRasBaudRate, lineGetDevCaps failed, Error:%x\n"), lReturn);
        break;
      }

      if (lpLineDevCaps->dwNeededSize <= lpLineDevCaps->dwTotalSize)
        break;
   
      // Buffer wasn't big enough.  Reallocate.
      dwSize = lpLineDevCaps->dwNeededSize;
      delete [] lpLineDevCaps;
      lpLineDevCaps = (LPLINEDEVCAPS) new BYTE[dwSize];
      lpLineDevCaps->dwTotalSize = dwSize;
    }
    if (lReturn)  // Failed to lineGetDevCaps
      continue;

    LPTSTR lpszTapiDeviceName = (LPTSTR) ((LPBYTE) lpLineDevCaps + lpLineDevCaps->dwLineNameOffset);

    //Is this TAPI line device the one RAS is using?
    if (lstrcmp(sDeviceName, lpszTapiDeviceName) != 0)
      continue;

    //Have to open the line to get any possible call handles
    HLINE hLine;
    lReturn = lineOpen(m_hLineApp, dwDeviceID, &hLine, dwApiVersion, 0, 0, LINECALLPRIVILEGE_NONE, 0, 0);
    if (lReturn)
    {
      //Failed to open.  RAS can't be using this line.
      TRACE(_T("CRasMonitor::GetRasBaudRate, lineOpen failed, Error:%x\n"), lReturn);
      continue;
    }

    // Now get all outstanding call handles
    while (TRUE)
    {
      lReturn = lineGetNewCalls(hLine, 0, LINECALLSELECT_LINE, lpCallList);
      if (lReturn)
      {
        // If this fails, something is seriously wrong, but RAS
        // is very unlikely to be using this line.
        TRACE(_T("CRasMonitor::GetRasBaudRate, lineGetNewCalls failed, Error:%x\n"), lReturn);
        break;
      }

      if (lpCallList->dwNeededSize <= lpCallList->dwTotalSize)
        break;

      //buffer wasn't big enough (not likely to happen, but could).
      //Reallocate and try again.
      DWORD dwSize = lpCallList->dwNeededSize;
      delete [] lpCallList;
      lpCallList = (LPLINECALLLIST) new BYTE[dwSize];
      lpCallList->dwTotalSize = dwSize;
    }

    if (lReturn)  //Failed to get the new call list.
      continue;

    DWORD dwNumCalls = lpCallList->dwCallsNumEntries;
    HCALL* lpCalls = (HCALL*) ((LPBYTE)lpCallList + lpCallList->dwCallsOffset);

    //No calls on this line?  Try the next line.
    if (dwNumCalls == 0)
       continue;

    //Identify which call RAS is using.

    //ASSUMPTION:  The speed of the first call is good enough.
    //If a modem is used, only one call per line is possible.
    //If ISDN is used, all calls on the line are the same speed.
    //Modem banks will be represented as one modem per line.
    hCall = lpCalls[0];

    break;
  }

  //Did we find the TAPI call of interest?
  if (hCall)
  {
    DWORD dwLineCallInfoSize = sizeof(LINECALLINFO) + 2048;
    LINECALLINFO* pLineCallInfo = (LINECALLINFO*) _alloca(dwLineCallInfoSize);
    pLineCallInfo->dwTotalSize = dwLineCallInfoSize;
    long lReturn = lineGetCallInfo(hCall, pLineCallInfo);
    if (lReturn)
    {
      // If this fails, something is seriously wrong, but RAS
      // is very unlikely to be using this line.
      TRACE(_T("CRasMonitor::GetRasBaudRate, lineGetCallInfo failed, Error:%x\n"), lReturn);
      hCall = 0;
    }

    //Fill in the details
    if (bGetConnectionSpeed)
      connection.m_dwConnectionSpeed = pLineCallInfo->dwRate;

    //Caller ID Number    
    if (pLineCallInfo->dwCallerIDSize && pLineCallInfo->dwCallerIDOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCallerIDSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCallerIDOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCallerIDSize);
      pszTempDest[pLineCallInfo->dwCallerIDSize] = '\0';
      connection.m_sCallerIDNumber = pszTempDest;
    }

    //Caller ID Name
    if (pLineCallInfo->dwCallerIDNameSize && pLineCallInfo->dwCallerIDNameOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCallerIDNameSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCallerIDNameOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCallerIDNameSize);
      pszTempDest[pLineCallInfo->dwCallerIDNameSize] = '\0';
      connection.m_sCallerIDName = pszTempDest;
    }

    //Called ID Number    
    if (pLineCallInfo->dwCalledIDSize && pLineCallInfo->dwCalledIDOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCalledIDSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCalledIDOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCalledIDSize);
      pszTempDest[pLineCallInfo->dwCalledIDSize] = '\0';
      connection.m_sCalledIDNumber = pszTempDest;
    }

    //Called ID Name
    if (pLineCallInfo->dwCalledIDNameSize && pLineCallInfo->dwCalledIDNameOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCalledIDNameSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCalledIDNameOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCalledIDNameSize);
      pszTempDest[pLineCallInfo->dwCalledIDNameSize] = '\0';
      connection.m_sCalledIDName = pszTempDest;
    }

    //Connected ID Number    
    if (pLineCallInfo->dwConnectedIDSize && pLineCallInfo->dwConnectedIDOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwConnectedIDSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwConnectedIDOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwConnectedIDSize);
      pszTempDest[pLineCallInfo->dwConnectedIDSize] = '\0';
      connection.m_sConnectedIDNumber = pszTempDest;
    }

    //Connected ID Name
    if (pLineCallInfo->dwConnectedIDNameSize && pLineCallInfo->dwConnectedIDNameOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwConnectedIDNameSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwConnectedIDNameOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwConnectedIDNameSize);
      pszTempDest[pLineCallInfo->dwConnectedIDNameSize] = '\0';
      connection.m_sConnectedIDName = pszTempDest;
    }

    //Displayable address
    if (pLineCallInfo->dwDisplayableAddressSize && pLineCallInfo->dwDisplayableAddressOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwDisplayableAddressSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwDisplayableAddressOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwDisplayableAddressSize);
      pszTempDest[pLineCallInfo->dwDisplayableAddressSize] = '\0';
      connection.m_sDisplayableAddress = pszTempDest;
    }

    //Called Party
    if (pLineCallInfo->dwCalledPartySize && pLineCallInfo->dwCalledPartyOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCalledPartySize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCalledPartyOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCalledPartySize);
      pszTempDest[pLineCallInfo->dwCalledPartySize] = '\0';
      connection.m_sCalledParty = pszTempDest;
    }

    //Comment
    if (pLineCallInfo->dwCommentSize && pLineCallInfo->dwCommentOffset)
    {
      LPSTR pszTempDest = (LPSTR) _alloca(pLineCallInfo->dwCommentSize + 1);
      LPCSTR pszTempSource = (LPCSTR) (((BYTE*) pLineCallInfo) + pLineCallInfo->dwCommentOffset);
      CopyMemory(pszTempDest, pszTempSource, pLineCallInfo->dwCommentSize);
      pszTempDest[pLineCallInfo->dwCommentSize] = '\0';
      connection.m_sComment = pszTempDest;
    }
  }

  //Tidy up the heap memory we have used
  delete [] lpCallList;
  delete [] lpLineDevCaps;

  if (hCall == 0)
    return FALSE;

  return TRUE;
}

void CALLBACK CRasMonitor::lineCallbackFunc(DWORD /*dwDevice*/, DWORD /*dwMsg*/, DWORD /*dwCallbackInstance*/, DWORD /*dwParam1*/, DWORD /*dwParam2*/, DWORD /*dwParam3*/)
{
   // Not doing anything in the callback.
}
#endif //RAS_MONITOR_NO_TAPI

#ifndef RAS_MONITOR_NO_PDH
CString CRasMonitor::GetPDHTotalBytesTxCounterName()
{
  return CString(_T("\\Ras Total\\Bytes Transmitted"));
}

CString CRasMonitor::GetPDHTotalBytesRxCounterName()
{
  return CString(_T("\\Ras Total\\Bytes Received"));
}
#endif

#ifndef RAS_MONITOR_NO_DYNDATA
CString CRasMonitor::GetDynDataTotalBytesRecvdCounterName()
{
  return _T("TotalBytesRecvd");
}

CString CRasMonitor::GetDynDataTotalBytesSentCounterName()
{
  return _T("TotalBytesXmit");
}

CString CRasMonitor::GetDynDataCounterName()
{
  return _T("Dial-Up Adapter");
}
#endif

UINT CRasMonitor::GetTimerID()
{
  return 1;
}